#ifndef __COREINIT_H_INCLUDED__
#define __COREINIT_H_INCLUDED__
// stand includes
#include <stdint-gcc.h>
// esp components includes
#include <esp_attr.h>
// project includes
// this header or bsp_config.h proj_config.h


#define CONFIG_MODULE_NAME_LEN      8


#ifdef __cplusplus
extern "C" {
#endif


// @brief 定义各个模块
typedef struct init_desc {
    char name[CONFIG_MODULE_NAME_LEN];
    int priority;       // 任务优先级：0为最低
    int core;           // 运行在哪个核, 0, 1, tskNO_AFFINITY:不指定
    int stack_size;     // 堆栈大小，单位字节
    int (*init)(struct init_desc *self);
    void (*thread)(struct init_desc *self);
    void* priv_data;     // 一些参数
} init_desc_t;

/** @brief 通过该宏实现定义一个初始化描述变量，初始化系统会根据设置的参数进行按顺序初始化和启动
 *  @param nm           模块名称
 *  @param priv         本模块私有的一些数据的指针
 *  @param level        模块级别，字符串，字符串排序越小越先初始化和执行
 *  @param init_func    初始化函数，类型为：int (*)(init_desc_t *self)
 *  @param run_core     运行在哪个核, tskNO_AFFINITY:不指定
 *  @param run_stack_size   堆栈大小，单位字节
 *  @param thread_priority  线程优先级 tskIDLE_PRIORITY 到 configMAX_PRIORITIES-1
 *  @param thread_func
*/
#define INIT_FN_EXPORT(nm, priv, level, init_func, run_core, run_stack_size, thread_priority, thread_func) \
    static __attribute__((section(".init_fn" level),used)) \
        init_desc_t init_fn_##nm = { \
            .name = #nm, \
            .priority = (thread_priority), \
            .core = (run_core), \
            .stack_size = (run_stack_size), \
            .init = (init_func), \
            .thread = (thread_func), \
            .priv_data = (priv), \
        };

#ifdef __cplusplus
}
#endif

#endif
